@@ -4,24 +4,36 @@ import sys
 import signal
 import socket
 import logging
+from time import time
 from uuid import uuid1
-from types import MethodType
 from datetime import datetime
+from functools import wraps
 from contextlib import contextmanager
 
 from kuyruk import events, importer
 from kuyruk.queue import Queue
 from kuyruk.events import EventMixin
-from kuyruk.helpers import profile
 from kuyruk.exceptions import Timeout
 
 logger = logging.getLogger(__name__)
 
 
+def profile(f):
+    """Logs the time spent while running the task."""
+    @wraps(f)
+    def inner(self, *args, **kwargs):
+        start = time()
+        result = f(self, *args, **kwargs)
+        end = time()
+        logger.info("%s finished in %i seconds." % (self.name, end - start))
+        return result
+    return inner
+
+
 class Task(EventMixin):
 
-    def __init__(self, f, kuyruk, queue='kuyruk', local=False,
-                 eager=False, retry=0, max_run_time=None):
+    def __init__(self, f, kuyruk, queue='kuyruk', local=False, eager=False,
+                 retry=0, max_run_time=None, arg_class=None):
         self.f = f
         self.kuyruk = kuyruk
         self.queue = queue
@@ -30,6 +42,7 @@ class Task(EventMixin):
         self.retry = retry
         self.max_run_time = max_run_time
         self.cls = None
+        self.arg_class = arg_class
         self.setup()
 
     def setup(self):
@@ -47,15 +60,22 @@ class Task(EventMixin):
         without changing the client code.
 
         """
+        logger.debug("Task.__call__ args=%r, kwargs=%r", args, kwargs)
         self.send_signal(events.task_presend, args, kwargs, reverse=True)
 
-        task_result = TaskResult(self)
+        # These keyword argument allow the sender to override
+        # the destination of the message.
+        host = kwargs.pop('kuyruk_host', None)
+        local = kwargs.pop('kuyruk_local', None)
 
         if self.eager or self.kuyruk.config.EAGER:
-            task_result.result = self.apply(*args, **kwargs)
+            # Run the task in process
+            task_result = self.apply(*args, **kwargs)
         else:
-            host = kwargs.pop('kuyruk_host', None)
-            task_result.id = self.send_to_queue(args, kwargs, host=host)
+            # Send it to the queue
+            task_result = TaskResult(self)
+            task_result.id = self.send_to_queue(
+                args, kwargs, host=host, local=local)
 
         self.send_signal(events.task_postsend, args, kwargs)
 
@@ -76,10 +96,15 @@ class Task(EventMixin):
         """
         self.cls = objtype
         if obj:
-            return MethodType(self.__call__, obj, objtype)
+            # Class tasks needs to know what the object is so they can
+            # inject that object in front of args.
+            # We are returning a BoundTask instance here wrapping this task
+            # that will do the injection.
+            logger.debug("Creating bound task with obj=%r", obj)
+            return BoundTask(self, obj)
         return self
 
-    def send_to_queue(self, args, kwargs, host=None):
+    def send_to_queue(self, args, kwargs, host=None, local=None):
         """
         Sends this task to queue.
 
@@ -88,28 +113,37 @@ class Task(EventMixin):
             on execution.
         :param host: Send this task to specific host. ``host`` will be
             appended to the queue name.
+        :param local: Send this task to this host. Hostname of this host will be
+            appended to the queue name.
         :return: :const:`None`
 
         """
+        logger.debug("Task.send_to_queueue args=%r, kwargs=%r", args, kwargs)
+        queue = self.queue
+        local_ = self.local
+
+        if local is not None:
+            local_ = local
+
         if host:
             queue = "%s.%s" % (self.queue, host)
-            local = False
-        else:
-            queue = self.queue
-            local = self.local
+            local_ = False
 
         with self.kuyruk.channel() as channel:
-            queue = Queue(queue, channel, local)
+            queue = Queue(queue, channel, local_)
             desc = self.get_task_description(args, kwargs, queue.name)
             queue.send(desc)
 
+        # We are returning the unique identifier of the task sent to queue
+        # so we can query the result backend for completion.
+        # TODO no result backend is available yet
         return desc['id']
 
     def get_task_description(self, args, kwargs, queue):
         """Return the dictionary to be sent to the queue."""
 
         # For class tasks; replace the first argument with the id of the object
-        if self.cls:
+        if self.cls or self.arg_class:
             args = list(args)
             args[0] = args[0].id
 
@@ -147,6 +181,10 @@ class Task(EventMixin):
         def send_signal(signal, reverse=False, **extra):
             self.send_signal(signal, args, kwargs, reverse, **extra)
 
+        logger.debug("Task.apply args=%r, kwargs=%r", args, kwargs)
+
+        result = TaskResult(self)
+
         limit = (self.max_run_time or
                  self.kuyruk.config.MAX_TASK_RUN_TIME or 0)
 
@@ -161,9 +199,15 @@ class Task(EventMixin):
             raise
         else:
             send_signal(events.task_success, return_value=return_value)
+            result.result = return_value
         finally:
             send_signal(events.task_postrun)
 
+        # We are returning a TaskResult here because __call__ returns a
+        # TaskResult object too. Return value must be consistent whether
+        # task is sent to queue or executed in process with apply().
+        return result
+
     @property
     def name(self):
         """Location for the wrapped function.
@@ -192,6 +236,40 @@ class Task(EventMixin):
             return self.cls.__name__
 
 
+class BoundTask(Task):
+    """
+    This class wraps the Task and inject the bound object in front of args
+    when it is called.
+
+    """
+    def __init__(self, task, obj):
+        self.task = task
+        self.obj = obj
+
+    def __getattr__(self, item):
+        """Delegates all attributes to real Task."""
+        return getattr(self.task, item)
+
+    @wraps(Task.__call__)
+    def __call__(self, *args, **kwargs):
+        logger.debug("BoundTask.__call__ args=%r, kwargs=%r", args, kwargs)
+        # Insert the bound object as a first argument to __call__
+        args = list(args)
+        args.insert(0, self.obj)
+        return super(BoundTask, self).__call__(*args, **kwargs)
+
+    @wraps(Task.apply)
+    def apply(self, *args, **kwargs):
+        logger.debug("BoundTask.apply args=%r, kwargs=%r", args, kwargs)
+        # apply() may be called directly. Insert the bound object only if
+        # it is not inserted by __call__()
+        if args and args[0] is not self.obj:
+            args = list(args)
+            args.insert(0, self.obj)
+
+        return super(BoundTask, self).apply(*args, **kwargs)
+
+
 class TaskResult(object):
     """Insance of this class is returned after the task is sent to queue.
     Since Kuyruk does not support a result backend yet it will raise
