@@ -1,13 +1,13 @@
 import signal
-import inspect
 import logging
 import unittest
 
 import redis
+from mock import patch
 
 import tasks
 from kuyruk import Task
-from kuyruk.task import TaskResult
+from kuyruk.task import TaskResult, BoundTask
 from util import *
 
 logger = logging.getLogger(__name__)
@@ -17,7 +17,13 @@ logger.debug('Process group id: %s', os.getpgrp())
 
 
 class KuyrukTestCase(unittest.TestCase):
+    """
+    Tests here are mostly integration tests. They require a running
+    RabbitMQ and Redis instances to pass succesfully.
+    Just like the normal user they spawn a real master/worker process
+    and make some assertion in their output.
 
+    """
     def setUp(self):
         delete_queue('kuyruk')
 
@@ -71,11 +77,19 @@ class KuyrukTestCase(unittest.TestCase):
             master.expect_exit(0)
             wait_until(not_running, timeout=TIMEOUT)
 
-    def test_eager(self):
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_eager(self, mock_func):
         """Test eager mode for using in test environments"""
         result = tasks.eager_task()
         assert isinstance(result, TaskResult)
-        self.assertTrue(tasks.eager_called)
+        mock_func.assert_called_once_with()
+
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_apply(self, mock_func):
+        """Test Task.apply()"""
+        result = tasks.print_task.apply("hello")
+        assert isinstance(result, TaskResult)
+        mock_func.assert_called_once_with()
 
     def test_reject(self):
         """Rejected tasks must be requeued again"""
@@ -115,7 +129,6 @@ class KuyrukTestCase(unittest.TestCase):
 
     def test_save_failed(self):
         """Failed tasks are saved to Redis"""
-        delete_queue('kuyruk_failed')
         tasks.raise_exception()
         with run_kuyruk(save_failed_tasks=True) as master:
             master.expect('ZeroDivisionError')
@@ -132,6 +145,42 @@ class KuyrukTestCase(unittest.TestCase):
         assert not r.hvals('failed_tasks')
         assert not is_empty('kuyruk')
 
+    def test_save_failed_class_task(self):
+        """Failed tasks are saved to Redis"""
+        cat = tasks.Cat(1, 'Felix')
+
+        cat.raise_exception()
+        with run_kuyruk(save_failed_tasks=True) as master:
+            master.expect('raise Exception')
+            master.expect('Saving failed task')
+            master.expect('Saved')
+
+        assert is_empty('kuyruk')
+        r = redis.StrictRedis()
+        assert r.hvals('failed_tasks')
+
+        run_requeue()
+        assert not r.hvals('failed_tasks')
+        assert not is_empty('kuyruk')
+
+    def test_save_failed_arg_class(self):
+        """Failed tasks are saved to Redis"""
+        cat = tasks.Cat(1, 'Felix')
+
+        tasks.jump_fail(cat)
+        with run_kuyruk(save_failed_tasks=True) as master:
+            master.expect('ZeroDivisionError')
+            master.expect('Saving failed task')
+            master.expect('Saved')
+
+        assert is_empty('kuyruk')
+        r = redis.StrictRedis()
+        assert r.hvals('failed_tasks')
+
+        run_requeue()
+        assert not r.hvals('failed_tasks')
+        assert not is_empty('kuyruk')
+
     def test_dead_master(self):
         """If master is dead worker should exit gracefully"""
         tasks.print_task('hello world')
@@ -141,10 +190,11 @@ class KuyrukTestCase(unittest.TestCase):
             master.expect_exit(-signal.SIGKILL)
             wait_until(not_running, timeout=TIMEOUT)
 
-    def test_before_after(self):
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_before_after(self, mock_func):
         """Before and after task functions are run"""
         tasks.task_with_functions('hello world')
-        self.assertTrue(tasks.presend_called)
+        mock_func.assert_called_once_with()
         with run_kuyruk() as master:
             master.expect('function1')
             master.expect('function2')
@@ -163,16 +213,42 @@ class KuyrukTestCase(unittest.TestCase):
     def test_class_task(self):
         cat = tasks.Cat(1, 'Felix')
         self.assertTrue(isinstance(tasks.Cat.meow, Task))
-        self.assertTrue(inspect.ismethod(cat.meow))
+        self.assertTrue(isinstance(cat.meow, BoundTask))
 
-    def test_class_task_fail(self):
+        cat.meow('Oh my god')
+        with run_kuyruk() as master:
+            master.expect('Oh my god')
+
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_class_task_eager(self, mock_func):
         cat = tasks.Cat(1, 'Felix')
+        cat.meow_eager('Oh my god')
+        mock_func.assert_called_once_with()
 
-        cat.raise_exception()
-        with run_kuyruk(save_failed_tasks=True) as master:
-            master.expect('raise Exception')
-            master.expect('Saving failed task')
-            master.expect('Saved')
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_class_task_apply(self, mock_func):
+        cat = tasks.Cat(1, 'Felix')
+        cat.meow.apply('Oh my god')
+        mock_func.assert_called_once_with()
+
+    def test_arg_class(self):
+        cat = tasks.Cat(1, 'Felix')
+        tasks.jump(cat)
+        with run_kuyruk() as master:
+            master.expect('Felix jumps high!')
+            master.expect('Called with Felix')
+
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_arg_class_eager(self, mock_func):
+        cat = tasks.Cat(1, 'Felix')
+        tasks.jump_eager(cat)
+        mock_func.assert_called_once_with('Felix')
+
+    @patch('kuyruk.test.tasks.must_be_called')
+    def test_arg_class_apply(self, mock_func):
+        cat = tasks.Cat(1, 'Felix')
+        tasks.jump.apply(cat)
+        mock_func.assert_called_once_with('Felix')
 
     def test_task_name(self):
         self.assertEqual(tasks.Cat.meow.name, 'kuyruk.test.tasks:Cat.meow')
